#!/usr/bin/python
# -*- coding: UTF-8 -*-
#
# `Damo-Embedding` - 'c++ tool for sparse parameter server'
# Copyright (C) 2019 - present timepi <timepi123@gmail.com>
# `Damo-Embedding` is provided under: GNU Affero General Public License
# (AGPL3.0) https:#www.gnu.org/licenses/agpl-3.0.html unless stated otherwise.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
import unittest
from test import support
import damo
import numpy as np


class SGDTestCase(unittest.TestCase):
    def setUp(self):
        self.storage = damo.PyStorage("/tmp/data_dir", 86400)
        init_params = damo.Parameters()
        init_params.insert("name", "truncate_normal")
        init_params.insert("mean", 0.0)
        init_params.insert("stddev", 1.0)
        self.initializer = damo.PyInitializer(init_params)
        optm_params = damo.Parameters()
        optm_params.insert("name", "sgd")
        self.lr = 0.001
        optm_params.insert("gamma", self.lr)
        optm_params.insert("lambda", 0.0)
        self.optimizer = damo.PyOptimizer(optm_params)

        self.dim = 16
        group = 0
        self.embedding = damo.PyEmbedding(
            self.storage, self.optimizer, self.initializer, self.dim, group
        )

    def tearDown(self):
        pass

    def test(self):
        n = 8
        keys = np.zeros(n, dtype=np.uint64)
        for i in range(n):
            keys[i] = i + 1
        w = np.zeros(self.dim * keys.shape[0], dtype=np.float32)
        gds = np.random.random(self.dim * keys.shape[0]).astype(np.float32)
        self.embedding.lookup(keys, w)
        a = w - self.lr * gds
        self.embedding.apply_gradients(keys, gds)
        self.embedding.lookup(keys, w)
        assert np.linalg.norm(a - w) == 0.0


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
# -*- coding: UTF-8 -*-
#
# `Damo-Embedding` - 'c++ tool for sparse parameter server'
# Copyright (C) 2019 - present timepi <timepi123@gmail.com>
# `Damo-Embedding` is provided under: GNU Affero General Public License
# (AGPL3.0) https:#www.gnu.org/licenses/agpl-3.0.html unless stated otherwise.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
import unittest
import damo


class CountBloomFilterTestCase(unittest.TestCase):
    def setUp(self):
        params = damo.Parameters()
        self.filter = damo.PyFilter(params)

    def tearDown(self):
        pass

    def testAdd(self):
        key = 10000000

        for i in range(16):
            self.filter.add(1, key, 1)
            print(self.filter.check(1, key))
            print(self.filter.check(2, key))


if __name__ == "__main__":
    unittest.main()



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.1
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _damo
else:
    import _damo

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class Parameters(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _damo.Parameters_swiginit(self, _damo.new_Parameters(*args))
    __swig_destroy__ = _damo.delete_Parameters

    def insert(self, *args):
        return _damo.Parameters_insert(self, *args)

    def to_json(self):
        return _damo.Parameters_to_json(self)
    params_ = property(_damo.Parameters_params__get, _damo.Parameters_params__set)

# Register Parameters in _damo:
_damo.Parameters_swigregister(Parameters)

class PyInitializer(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _damo.PyInitializer_swiginit(self, _damo.new_PyInitializer(*args))

    def call(self, w):
        return _damo.PyInitializer_call(self, w)
    __swig_destroy__ = _damo.delete_PyInitializer

# Register PyInitializer in _damo:
_damo.PyInitializer_swigregister(PyInitializer)

class PyOptimizer(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _damo.PyOptimizer_swiginit(self, _damo.new_PyOptimizer(*args))

    def call(self, w, gds, global_step=0):
        return _damo.PyOptimizer_call(self, w, gds, global_step)
    __swig_destroy__ = _damo.delete_PyOptimizer

# Register PyOptimizer in _damo:
_damo.PyOptimizer_swigregister(PyOptimizer)

class PyFilter(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _damo.PyFilter_swiginit(self, _damo.new_PyFilter(*args))

    def check(self, group, key):
        return _damo.PyFilter_check(self, group, key)

    def add(self, group, key, num):
        return _damo.PyFilter_add(self, group, key, num)
    __swig_destroy__ = _damo.delete_PyFilter

# Register PyFilter in _damo:
_damo.PyFilter_swigregister(PyFilter)

class PyStorage(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, data_dir, ttl=0):
        _damo.PyStorage_swiginit(self, _damo.new_PyStorage(data_dir, ttl))
    __swig_destroy__ = _damo.delete_PyStorage

    def dump(self, *args):
        return _damo.PyStorage_dump(self, *args)

    def checkpoint(self, path):
        return _damo.PyStorage_checkpoint(self, path)

    def load_from_checkpoint(self, path):
        return _damo.PyStorage_load_from_checkpoint(self, path)

# Register PyStorage in _damo:
_damo.PyStorage_swigregister(PyStorage)

class PyEmbedding(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _damo.PyEmbedding_swiginit(self, _damo.new_PyEmbedding(*args))
    __swig_destroy__ = _damo.delete_PyEmbedding

    def lookup(self, keys, w):
        return _damo.PyEmbedding_lookup(self, keys, w)

    def apply_gradients(self, keys, gds):
        return _damo.PyEmbedding_apply_gradients(self, keys, gds)

# Register PyEmbedding in _damo:
_damo.PyEmbedding_swigregister(PyEmbedding)






#!/usr/bin/python
# -*- coding: UTF-8 -*-
#
# `Damo-Embedding` - 'c++ tool for sparse parameter server'
# Copyright (C) 2019 - present timepi <timepi123@gmail.com>
# `Damo-Embedding` is provided under: GNU Affero General Public License
# (AGPL3.0) https:#www.gnu.org/licenses/agpl-3.0.html unless stated otherwise.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
import unittest
from test import support
import damo
import numpy as np
import os
import shutil


class LionTestCase(unittest.TestCase):
    def setUp(self):
        if os.path.exists("/tmp/data_dir"):
            shutil.rmtree("/tmp/data_dir")
        self.storage = damo.PyStorage("/tmp/data_dir", 86400)
        init_params = damo.Parameters()
        init_params.insert("name", "truncate_normal")
        init_params.insert("mean", 0.0)
        init_params.insert("stddev", 1.0)
        self.initializer = damo.PyInitializer(init_params)
        optm_params = damo.Parameters()
        optm_params.insert("name", "lion")
        self.eta = 0.003
        self.beta1 = 0.9
        self.beta2 = 0.99
        self.lambda_ = 0.01
        optm_params.insert("eta", self.eta)
        optm_params.insert("beta1", self.beta1)
        optm_params.insert("beta2", self.beta2)
        optm_params.insert("lambda", self.lambda_)
        self.optimizer = damo.PyOptimizer(optm_params)

        self.dim = 16
        group = 0
        self.embedding = damo.PyEmbedding(
            self.storage, self.optimizer, self.initializer, self.dim, group
        )

    def tearDown(self):
        pass

    def test(self):
        n = 8
        keys = np.random.randint(1, 10000 + 1, n, dtype=np.uint64)
        w = np.zeros(self.dim * n, dtype=np.float32)
        self.embedding.lookup(keys, w)
        m = np.zeros(self.dim * n).astype(np.float32)
        gds = np.random.random(self.dim * n).astype(np.float32)
        tmp_mu = np.sign(self.beta1 * m + (1.0 - self.beta1) * gds) + w * self.lambda_
        m = self.beta2 * +(1.0 - self.beta2) * gds
        self.embedding.apply_gradients(keys, gds)
        new_w = w - self.eta * tmp_mu
        self.embedding.lookup(keys, w)

        assert np.linalg.norm(new_w - w) == 0.0


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# `Damo-Embedding` - 'c++ tool for sparse parameter server'
# Copyright (C) 2019 - present timepi <timepi123@gmail.com>
# `Damo-Embedding` is provided under: GNU Affero General Public License
# (AGPL3.0) https:#www.gnu.org/licenses/agpl-3.0.html unless stated otherwise.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#

import platform
import sys

from setuptools import Extension, setup, find_packages

COMPILE_OPTIONS = []
LINK_OPTIONS = []


def is_new_osx():
    """Check whether we're on OSX >= 10.7"""
    if sys.platform != "darwin":
        return False
    mac_ver = platform.mac_ver()[0]
    ver_ss = mac_ver.split(".")
    version = int(ver_ss[0])

    if version > 10:
        return True
    elif version == 10:
        minor_version = int(ver_ss[1])
        return minor_version >= 7
    else:
        return False


if is_new_osx():
    COMPILE_OPTIONS.append("-stdlib=libc++")
    COMPILE_OPTIONS.append("-std=c++17")
    LINK_OPTIONS.append("-lc++")
    LINK_OPTIONS.append("-lrocksdb")
    LINK_OPTIONS.append("-nodefaultlibs")
else:
    COMPILE_OPTIONS.append("-std=c++17")
    LINK_OPTIONS.append("-lrocksdb")
    LINK_OPTIONS.append("-lpthread")
    LINK_OPTIONS.append("-Wl,-rpath=/usr/local/lib")


class get_numpy_include(object):
    """Defer numpy.get_include() until after numpy is installed."""

    def __str__(self):
        import numpy

        return numpy.get_include()


damoModule = Extension(
    name="_damo",
    include_dirs=["include", get_numpy_include()],
    sources=[
        "src/pyembedding.cpp",
        "damo_wrap.cxx",
        "src/learning_rate_scheduler.cpp",
        "src/initializer.cpp",
        "src/embedding.cpp",
        "src/optimizer.cpp",
        "src/counting_bloom_filter.cpp",
        "src/common.cpp",
    ],
    extra_compile_args=COMPILE_OPTIONS,
    extra_link_args=LINK_OPTIONS,
)

with open("README.md", "r", encoding="utf-8") as fd:
    long_description = fd.read()

setup(
    name="damo-embedding",
    version="1.0.5",
    description="Python wrapper for damo, a set of fast and robust hash functions.",
    license="License :: AGLP3",
    author="timepi",
    author_email="",
    url="https://github.com/uopensail/damo-embedding",
    packages=find_packages(),
    py_modules=["damo"],
    ext_modules=[damoModule],
    keywords=["sparse embedding using rocksdb","parameter server","ftrl","adam","adagrad"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=["numpy>=1.19.0"],
    setup_requires=["numpy>=1.19.0"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
    ],
)
